==> iql-1.8.5.tar.gz: iql-1.8.5/iql/jupyter_magics/iql_magic.py <==
from IPython.core.getipython import get_ipython
import iql

import logging
import argparse
from typing import Optional, Iterable

try:
    from jinja2 import Template
except Exception:  # pragma: no cover
    # Nothing to do, jinja2 is optional
    # User will get an error when using -j
    pass

from traitlets.config.configurable import Configurable
from IPython.core.magic_arguments import argument, magic_arguments, parse_argstring
from IPython.core.magic import (
    Magics,
    cell_magic,
    line_magic,
    magics_class,
    no_var_expand,
    needs_local_scope,
)

from duckdb import DuckDBPyConnection

logger = logging.getLogger(__name__)

connection: Optional[DuckDBPyConnection] = None


def _get_obj_from_name(name: str) -> Optional[object]:
    ip = get_ipython()
    return ip.ev(name) if ip is not None else None


def execute_iql(
    query_string: str,
    connection: Optional[DuckDBPyConnection],
    export_function: Optional[str] = None,
    query_params: Optional[Iterable] = None,
):
    return iql.execute(query_string, connection, parameters=query_params)


@magics_class
class IqlMagic(Magics, Configurable):
    # database connection object
    # created via -d (default), -cn (connection string) or -co (connection object)

    # selected via -t. None = Pandas.
    export_function = None

    def __init__(self, shell):
        Configurable.__init__(self, config=shell.config)
        Magics.__init__(self, shell=shell)

        # Add ourself to the list of module configurable via %config
        self.shell.configurables.append(self)  # type: ignore

    def connect_by_objectname(self, connection_object):
        con: DuckDBPyConnection = _get_obj_from_name(connection_object)  # type: ignore
        if not isinstance(con, DuckDBPyConnection):
            raise ValueError(f"{connection_object} is not a DuckDBPyConnection")
        logger.debug(f"Using existing connection: {connection_object}")

        global connection
        connection = con

    @no_var_expand
    @needs_local_scope
    @line_magic("iql")
    @cell_magic("iql")
    @line_magic("bql")
    @cell_magic("bql")
    @magic_arguments()
    @argument(
        "-co",
        "--connection_object",
        help="Connect to a database using the connection object",
        nargs=1,
        type=str,
        action="store",
    )
    @argument(
        "-o",
        "--output",
        help="Write the output to the specified variable",
        nargs=1,
        type=str,
        action="store",
    )
    @argument(
        "-q",
        "--quiet",
        help="Don't return an object, similar to %%capture",
        action="store_true",
    )
    @argument("-j", "--jinja2", help="Apply Jinja2 Template", action="store_true")
    @argument(
        "-s",
        "--substitute",
        nargs=1,
        type=str,
        action="store",
        help="Substitute Variables using a Dictionary",
    )
    @argument(
        "-p",
        "--param",
        dest="queryparams",
        help="Params from user_ns",
        action="append",
    )
    @argument("rest", nargs=argparse.REMAINDER)
    def execute(self, line: str = "", cell: str = "", local_ns=None):
        global connection

        assert self.shell is not None

        args = parse_argstring(self.execute, line)
        # Grab rest of line
        rest = " ".join(args.rest)
        query = f"{rest}\n{cell}".strip()

        logger.debug("Query = %s", query)

        if args.substitute:
            substitutions = self.shell.user_ns[args.substitute[0]]
            for k, v in substitutions.items():
                query = query.replace(k, str(v))

        if args.jinja2:  # Replace any {var}'s with the string values
            query = Template(query).render(self.shell.user_ns)  # type: ignore

        if args.connection_object:
            self.connect_by_objectname(args.connection_object[0])

        if args.queryparams:
            query_params = [self.shell.user_ns[p] for p in args.queryparams]
        else:
            query_params = None

        if query is None or len(query) == 0:
            logger.debug("Nothing to execute")
            return

        o = execute_iql(
            query_string=query,
            connection=connection,
            export_function=self.export_function,
            query_params=query_params,
        )

        if args.output:
            self.shell.user_ns[args.output[0]] = o  # type: ignore

        if args.quiet:
            return None
        else:
            return o


def load_ipython_extension(ip):
    ip = get_ipython()

    ip.register_magics(IqlMagic)  # type: ignore

==> iql-1.8.8.tar.gz: iql-1.8.8/iql/jupyter_magics/iql_magic.py <==
import argparse
import logging
from typing import Iterable, Optional

from IPython.core.getipython import get_ipython

import iql

try:
    from jinja2 import Template
except Exception:  # noqa
    # Nothing to do, jinja2 is optional
    # User will get an error when using -j
    pass

from duckdb import DuckDBPyConnection
from IPython.core.magic import (
    Magics,
    cell_magic,
    line_magic,
    magics_class,
    needs_local_scope,
    no_var_expand,
)
from IPython.core.magic_arguments import argument, magic_arguments, parse_argstring
from traitlets.config.configurable import Configurable

logger = logging.getLogger(__name__)

connection: Optional[DuckDBPyConnection] = None


def _get_obj_from_name(name: str) -> Optional[object]:
    ip = get_ipython()
    return ip.ev(name) if ip is not None else None


def execute_iql(
    query_string: str,
    connection: Optional[DuckDBPyConnection],
    export_function: Optional[str] = None,
    query_params: Optional[Iterable] = None,
):
    return iql.execute(query_string, connection, parameters=query_params)


@magics_class
class IqlMagic(Magics, Configurable):
    # database connection object
    # created via -d (default), -cn (connection string) or -co (connection object)

    # selected via -t. None = Pandas.
    export_function = None
    shell = None

    def __init__(self, shell):
        Configurable.__init__(self, config=shell.config)
        Magics.__init__(self, shell=shell)

        # Add ourself to the list of module configurable via %config
        self.shell.configurables.append(self)  # type: ignore

    def connect_by_objectname(self, connection_object):
        con: DuckDBPyConnection = _get_obj_from_name(connection_object)  # type: ignore
        if not isinstance(con, DuckDBPyConnection):
            raise ValueError(f"{connection_object} is not a DuckDBPyConnection")
        logger.debug("Using existing connection: %s", connection_object)

        global connection
        connection = con

    @no_var_expand
    @needs_local_scope
    @line_magic("iql")
    @cell_magic("iql")
    @line_magic("bql")
    @cell_magic("bql")
    @magic_arguments()
    @argument(
        "-co",
        "--connection_object",
        help="Connect to a database using the connection object",
        nargs=1,
        type=str,
        action="store",
    )
    @argument(
        "-o",
        "--output",
        help="Write the output to the specified variable",
        nargs=1,
        type=str,
        action="store",
    )
    @argument(
        "-q",
        "--quiet",
        help="Don't return an object, similar to %%capture",
        action="store_true",
    )
    @argument("-j", "--jinja2", help="Apply Jinja2 Template", action="store_true")
    @argument(
        "-s",
        "--substitute",
        nargs=1,
        type=str,
        action="store",
        help="Substitute Variables using a Dictionary",
    )
    @argument(
        "-p",
        "--param",
        dest="queryparams",
        help="Params from user_ns",
        action="append",
    )
    @argument("rest", nargs=argparse.REMAINDER)
    def execute(self, line: str = "", cell: str = "", local_ns=None):
        global connection

        args = parse_argstring(self.execute, line)
        # Grab rest of line
        rest = " ".join(args.rest)
        query = f"{rest}\n{cell}".strip()

        logger.debug("Query = %s", query)

        if args.substitute:
            substitutions = self.shell.user_ns[args.substitute[0]]
            for k, v in substitutions.items():
                query = query.replace(k, str(v))

        if args.jinja2:  # Replace any {var}'s with the string values
            query = Template(query).render(self.shell.user_ns)  # type: ignore

        if args.connection_object:
            self.connect_by_objectname(args.connection_object[0])

        if args.queryparams:
            query_params = [self.shell.user_ns[p] for p in args.queryparams]
        else:
            query_params = None

        if query is None or len(query) == 0:
            logger.debug("Nothing to execute")
            return

        o = execute_iql(
            query_string=query,
            connection=connection,
            export_function=self.export_function,
            query_params=query_params,
        )

        if args.output:
            self.shell.user_ns[args.output[0]] = o  # type: ignore

        if args.quiet:
            return None
        else:
            return o
